/*   FILE: state-play.vala -- A state machine implementing lcddaap's menu
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 01 December 2013 
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using Gst;

class StatePlay : State {
	private MainLoop loop;
	private Gee.ArrayList<int> playlist;
	private int current = 0;
	private DMAP.Db db;

	public StatePlay (MainLoop loop, LCDPlayer player, DMAP.Db db, Gee.ArrayList<int> playlist) {
		this.loop     = loop;
		this.player   = player;
		this.db       = db;
		this.playlist = playlist;

		print_selected ();
	}

	// Print selected item.
	public override void print_selected () {
		// "Extra" index #0 is for "Play all"
		if (0 == current) {
			output ("Play all songs");
		} else {
			// Don't forget about "extra" index #0 (subtract 1 from current)
			output (((SimpleDAAPRecord) db.lookup_by_id (playlist[current - 1])).title);
		}
	}

	private bool bus_callback (Gst.Bus bus, Gst.Message message) {
                switch (message.type) {
                case MessageType.ERROR:
                        GLib.Error err;
                        string debug;
                        message.parse_error (out err, out debug);
                        GLib.stderr.printf ("Error: %s\n", err.message);
                        loop.quit ();
                        break;
                case MessageType.EOS:
                        loop.quit ();
                        break;
                default:
                        break;
                }

                return true;
        }

	// Select next menu item.
	public override void next () {
		current = (current + 1) % (playlist.size + 1);
                print_selected ();
	}

	// Select previous menu item.
	public override void previous () {
		current = (current + playlist.size) % (playlist.size + 1);
                print_selected ();
	}

	private void play (uint id) {
		SimpleDAAPRecord record = (SimpleDAAPRecord) db.lookup_by_id (id);

		var builder = new StringBuilder ();
		builder.append (record.location.substring (4, record.location.length - 4));
		builder.prepend ("http");

		player.element.uri = builder.str;

		Gst.Bus bus = player.element.get_bus ();
		bus.add_watch (GLib.Priority.DEFAULT, bus_callback);

		output ("Playing " + record.title);

		loop.quit ();
		player.element.set_state (Gst.State.PLAYING);

		register_handlers ();

		loop.run ();
		player.element.set_state (Gst.State.NULL);
	}

	// Transition to next state, based on current selection.
	public override State transition () {
		if (null != play) {
			player.element.set_state (Gst.State.NULL);
		}

		if (0 == current) {
			foreach (int i in playlist) {
				play (i);
			}
		} else {
			// Don't forget about "extra" index #0 (subtract 1 from current)
			play (playlist[current - 1]);
		}

		loop.run ();

		return this;
	}
}
