/*   FILE: state.vala -- A state machine implementing lcddaap's menu
 * AUTHOR: W. Michael Petullo <mike@flyn.org>
 *   DATE: 01 December 2013 
 *
 * Copyright (c) 2013 W. Michael Petullo <new@flyn.org>
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

enum SwitchNumber {
	SWITCH_PAUSE_PLAY = 3,
	SWITCH_ESCAPE     = 4,
	SWITCH_SELECT     = 5,
	SWITCH_LEFT       = 6,
	SWITCH_RIGHT      = 7
}

abstract class State : GLib.Object {
	private static bool initialized = false;
	uint lights_out_timeout_id = 0;

	// FIXME: Move button handlers to another class?
	private uint8[] lastSwitchState = { 1, 1, 1, 1, 1, 1, 1, 1 };

	protected unowned LCDPlayer player;

	// Print selected item.
	public abstract void print_selected ();

	// Select next menu item.
	public abstract void next ();

	// Select previous menu item.
	public abstract void previous ();

	// Transition to next state, based on current selection.
	public abstract State transition ();

	public State () {
		if (false == initialized) {
			register_handlers ();
			initialized = true;
		}
	}

	// Toggle between playing and paused.
	public void pause_play () {
		Gst.State state;
		player.element.get_state (out state, null, 0);
		if (Gst.State.PLAYING == state) {
			player.element.set_state (Gst.State.PAUSED);
			output ("Paused");
		} else if (Gst.State.PAUSED == state) {
			player.element.set_state (Gst.State.PLAYING);
			player.stateStack.peek ().print_selected ();
		}
	}

	public bool check_keyboard (IOChannel channel, IOCondition condition) {
		char []c = new char[1];

		if (condition == IOCondition.HUP) {
			GLib.stderr.printf ("Standard in closed?\n");
			return false;
		}

		try {
			size_t length = -1;
			IOStatus status = channel.read_chars (c, out length);
			if (status == IOStatus.EOF || length != 1) {
				GLib.stderr.printf ("Standard in closed?\n");
				return false;
			}
		} catch (IOChannelError e) {
			GLib.stderr.printf ("IOChannelError: %s\n", e.message);
			return false;
		} catch (ConvertError e) {
			GLib.stderr.printf ("ConvertError: %s\n", e.message);
			return false;
		}

                switch (c[0]) {
                case 'p':
			do_input (SwitchNumber.SWITCH_PAUSE_PLAY);
                        break;
                case 'a':
			do_input (SwitchNumber.SWITCH_LEFT);
                        break;
                case 's':
			do_input (SwitchNumber.SWITCH_RIGHT);
                        break;
                case '\n':
			do_input (SwitchNumber.SWITCH_SELECT);
                        break;
                case 'x':
			do_input (SwitchNumber.SWITCH_ESCAPE);
                        break;
                default:
			GLib.assert_not_reached ();
                }

                return true;
	}

	public void output (string str) {
                Pifacecad.lcd_clear ();

		if (str.length <= 15) {
			GLib.stdout.printf ("%s\n", str);
			Pifacecad.lcd_write (str);
		} else {
			var builder = new StringBuilder ();

			builder.append (str.substring (0, 16));
			builder.append ("\n  ");
			builder.append (str.substring (16, str.length <= 30 ? str.length - 16: 14));

			GLib.stdout.printf ("%s\n", str);
			Pifacecad.lcd_write (builder.str);
		}
	}

	protected bool lights_out () {
		GLib.debug ("Lights out.\n");
		Pifacecad.lcd_backlight_off ();
		this.lights_out_timeout_id = 0;
		return false;
	}

	protected void register_handlers () {
		try {
			IOChannel channel = new IOChannel.unix_new (Posix.STDIN_FILENO);
			channel.set_encoding (null);
			channel.add_watch (IOCondition.IN, check_keyboard);
			Timeout.add (SWITCH_POLL_MILLISECONDS, check_switches);
		} catch (IOChannelError e) {
			GLib.error ("Channel error initializing state");
		}
	}

	protected bool check_switches () {
		for (uint8 switch = 0; switch < 8; switch++) {
			uint8 current = Pifacecad.read_switch (switch);

			if (0 == current && 1 == lastSwitchState[switch]) {
				do_input (switch);
			}

			lastSwitchState[switch] = current;
		}

		return true;
	}

	private void do_input (uint8 op) {
		Pifacecad.lcd_backlight_on ();

		switch (op) {
		case SwitchNumber.SWITCH_PAUSE_PLAY:
			do_pause_play ();
			break;
		case SwitchNumber.SWITCH_LEFT:
			do_left ();
			break;
		case SwitchNumber.SWITCH_RIGHT:
			do_right ();
			break;
		case SwitchNumber.SWITCH_SELECT:
			do_select ();
			break;
		case SwitchNumber.SWITCH_ESCAPE:
			do_escape ();
			break;
		default:
			GLib.assert_not_reached ();
		}

		if (0 != lights_out_timeout_id) {
			Source.remove (lights_out_timeout_id);
		}
		lights_out_timeout_id = Timeout.add_seconds (LIGHTS_OUT_SECONDS, lights_out);
	}

	private void do_pause_play () {
		player.stateStack.peek ().pause_play ();
	}

	private void do_left () {
		player.stateStack.peek ().previous ();
	}

	private void do_right () {
		player.stateStack.peek ().next ();
	}

	private void do_select () {
		// Peek at top, call transition, push value returned (only push if new state).
		State next = player.stateStack.peek ().transition ();
		if (player.stateStack.peek () != next) {
			player.stateStack.push (next);
		}
	}

	private void do_escape () {
		if (player.stateStack.size () > 1) {
			player.stateStack.pop ();
		}
		player.stateStack.peek ().print_selected ();
	}
}
